import sys
import os.path
import shutil
import pyuppaal
###############################################################################################
###############################################################################################
cwd = os.path.dirname(os.path.realpath(__file__))
Base_property = cwd+"/baseData.xml"

## To generate TA sush that the property defined by the TA belongs to absence pattern. ##
## Generated TA defines property of form 
###"In any time interval of "timeConstraint" time units, there are no more than "n=complexity" "act1" actions.### 
def generateTA(complexity, act1, act2, timeConstraint):
    
    shutil.copy(Base_property, cwd+"/Example_Gen.xml")
    property = "Example_Gen.xml"
    # Current working directory
    projdir = os.path.normpath(os.path.join(os.path.dirname(__file__), '..'))
    # system path
    sys.path = [projdir] + sys.path

    Uppaal_Model = os.path.join(os.path.dirname(__file__), property)
    Parsed_NTA_Pyuppaal = pyuppaal.NTA.from_xml(open(Uppaal_Model))
    Property = Parsed_NTA_Pyuppaal.templates[0]
       
    Property.clocks=[]
    Property.chan=[]
    
    Property.add_Clock("x")
    Property.add_Chan(act1)
    Property.add_Chan(act2)
    
    currentLocation = Property.get_location_by_name("L0")
    ##self-transtions initial location
    initLocation = Property.get_location_by_name("L0")
    t1 = pyuppaal.Transition(initLocation, initLocation ,synchronisation = act2+"?")
    t2 = pyuppaal.Transition(initLocation, initLocation, guard = "x>="+str(timeConstraint), assignment = "x=0", synchronisation = act1+"?")
    Property.add_Transition(t1)
    Property.add_Transition(t2)
    
    
    #get the id of L0 in the run automaton
    locationIDcount = int((currentLocation.id)[-1])+1
    locationLableCount = 1
    locXpos = currentLocation.xpos
    locYpos = currentLocation.ypos
        
    for x in range(0,complexity):
        newLocation = pyuppaal.Location(name="L"+str(locationLableCount), id ="id" + str(locationIDcount), xpos = currentLocation.xpos+100 , ypos =currentLocation.ypos)
        locationIDcount= locationIDcount+1
        locationLableCount = locationLableCount+1
                
        Property.add_Location(newLocation)
        t1 = pyuppaal.Transition(currentLocation, newLocation,  guard = "x<"+str(timeConstraint), synchronisation = act1+"?")
        t2 = pyuppaal.Transition(newLocation, newLocation, synchronisation = act2+"?")
        t3 = pyuppaal.Transition(newLocation, initLocation, guard = "x>="+str(timeConstraint), assignment = "x=0", synchronisation = act1+"?")
        
        currentLocation = newLocation
        Property.add_Transition(t1)
        Property.add_Transition(t2)
        Property.add_Transition(t3)
    
    ##Final Location######
    finalLocation = pyuppaal.Location(name="Final_NA", id ="Final_NA" + str(locationIDcount), xpos = currentLocation.xpos+100 , ypos =currentLocation.ypos)
    t1 = pyuppaal.Transition(currentLocation, finalLocation,  guard = "x<"+str(timeConstraint), synchronisation = act1+"?")
    t2 = pyuppaal.Transition(finalLocation, finalLocation, synchronisation = act2+"?")
    t3 = pyuppaal.Transition(finalLocation, finalLocation, synchronisation = act1+"?")
    Property.add_Location(finalLocation)
    Property.add_Transition(t1)
    Property.add_Transition(t2)
    Property.add_Transition(t3)
    #########################
            
    pyuppaal.Template.to_xml(Property)
    f = open(Uppaal_Model, "r+")
    f.writelines(pyuppaal.NTA.to_xml(Parsed_NTA_Pyuppaal))
    f.close()

    print "TA is generated and is written to file Example_Gen.xml" 
##################################################################################################################
##################################################################################################################    
